# -*- coding: latin1 -*-
#############################################
#	OGR2Layers Plugin (c)  for Quantum GIS					#
#	(c) Copyright Nicolas BOZON - 2008					#
#	Authors: Nicolas BOZON, Rene-Luc D'HONT, Michael DOUCHIN	#
#	Email: nicolas_dot_bozon_at_gmail_dot_com				#
#	More info at www.ogr2layers.org						#
#############################################
#    OGR2Layers Plugin is licensed under the terms of GNU GPL 2		#
#  	This program is free software; you can redistribute it and/or modify	#
# 	 it under the terms of the GNU General Public License as published by	#
# 	 the Free Software Foundation; either version 2 of the License, or	#
# 	 (at your option) any later version.						#
#	This program is distributed in the hope that it will be useful,		#
#	 but WITHOUT ANY WARRANTY; without even implied warranty of	#
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	# 
# 	 See the GNU General Public License for more details.			#
#############################################

#Python, PyQt and QGIS imports
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from qgis.gui import *
from ogr2layersdialog import OGR2LayersDialog
import sys
import os
import resources

class OGR2Layers:
	MSG_BOX_TITLE = "OGR2Layers Plugin"
  
	def __init__(self, iface):
		self.iface = iface
	
	def initGui(self):
		self.action = QAction(QIcon(":/plugins/OGR2Layers/ogr2layersicon.xpm"), "OGR2Layers", self.iface.getMainWindow())
		self.action.setWhatsThis("Configuration for OGR2Layers plugin")
		QObject.connect(self.action, SIGNAL("activated()"), self.run)
		self.iface.addToolBarIcon(self.action)
		self.iface.addPluginMenu("&OGR2Layers Plugin...", self.action)
		QObject.connect(self.iface.getMapCanvas(), SIGNAL("renderComplete(QPainter *)"), self.renderTest)

	#Plugin disjunction
	def unload(self):
		self.iface.removePluginMenu("&OGR2Layers Plugin...",self.action)
		self.iface.removeToolBarIcon(self.action)
		QObject.disconnect(self.iface.getMapCanvas(), SIGNAL("renderComplete(QPainter *)"), self.renderTest)
 
	#Counts and validate vector layers 
	def getVectorLayerByName(self, myName):
		mc = self.iface.getMapCanvas()
		nLayers = mc.layerCount()
		for l in range(nLayers):

			layer = mc.getZpos(l)
			if layer.name() == unicode(myName,'latin1'):
				if layer.isValid():
					return layer
				else:
					QMessageBox.information(self, "OGR2Layers", "Vector layer is not valid")  
	
	#Checks vector validity and gets layer names and sources 
	def run(self):
		self.dlg = OGR2LayersDialog()
		QObject.connect(self.dlg.ui.browseButton, SIGNAL("clicked()"), self.SelectKmlDir)    
		layers =  self.iface.activeLayer()
		#Checks for loaded layers, do not load if no layers
		if layers == None:
			QMessageBox.warning(self.iface.getMainWindow(), self.MSG_BOX_TITLE, ("No active layer found\n" "Please make one or more OGR layer active\n" "Beware of layers sizes for export"), QMessageBox.Ok, QMessageBox.Ok)
			return
		self.layers = []     
		mapCanvas = self.iface.getMapCanvas()
		#Checks vector type and populates the layer list view
		for i in range(mapCanvas.layerCount()):
			layer = mapCanvas.getZpos(i)
			if layer.type() == layer.VECTOR:
				self.layers.append(layer)
				self.dlg.ui.LayerList.addItems(QStringList(layer.source()))
		QObject.connect(self.dlg.ui.okbutton, SIGNAL("clicked()"), self.WriteKML)
		#Set up the default map extent (world)
		Extent = mapCanvas.extent()
		mylayer = self.layers[0]
		myprovider = mylayer.getDataProvider()
		SrsSrc = myprovider.getSRS()
		SrsDest = QgsSpatialRefSys(4326)    # WGS 84
		xform = QgsCoordinateTransform(SrsSrc,SrsDest)
		minPt = xform.transform(QgsPoint(Extent.xMin(),Extent.yMin()))
		maxPt = xform.transform(QgsPoint(Extent.xMax(),Extent.yMax()))
		#QMessageBox.information(self.iface.getMainWindow(),"Information",str(minPt)   )
		xMin = self.dlg.ui.lineEdit.setText(str(minPt.x()))
		yMin = self.dlg.ui.lineEdit_2.setText(str(minPt.y()))
		xMax = self.dlg.ui.lineEdit_3.setText(str(maxPt.x()))
		yMax = self.dlg.ui.lineEdit_4.setText(str(maxPt.y()))
		settings = QSettings("3LIZ", "OGR2Layers")
		global mydir
		mydir=""
		self.dlg.show()
		result = self.dlg.exec_() 
		
	def SelectKmlDir(self):
		#set up the output dir for KML
		global mydir
		mydir = QFileDialog.getExistingDirectory( None,QString("Choose the kml files destination folder"),"" )
		self.dlg.ui.kmldirpath.setText(mydir)
		
	def WriteKML(self):
		layers = self.layers
		#configure ProgressBar
		self.dlg.ui.progressBar.setMinimum(0) 
		self.dlg.ui.progressBar.setMaximum(len(layers)) 
		compteur=0
		
		#control variable
		control =[]
		
		#test if the output dir "mydir" is set and correct
		if str(self.dlg.ui.kmldirpath.toPlainText())=='':
			control.append("Please select the output directory first")
		else:
			mydirtest=self.dlg.ui.kmldirpath.toPlainText()
			if not os.path.exists(mydirtest):
				control.append("Please check the validity of the output directory.")
		#test if there are some layers loaded in qgis
		if len(layers) == 0:
			control.append("No layer is loaded. Please load some layers, and restart the plugin")
		
		if len(control) == 0:
			for layer in layers:
				if layer and layer.type() == QgsMapLayer.VECTOR:
						compteur = compteur + 1
						
						myinfo=[]
						#source of each vector layer (postgres, shapefile path, etc.)
						mysource = layer.source()
						#layer name
						myname = layer.name()
						#layer provider (ogr, postgres, etc.)
						myprovider = layer.getDataProvider()
						myprovidername = myprovider.name()
						#layer EPSG projection
						mysrs = myprovider.getSRS() #layer spatial reference system
						myepsg = mysrs.epsg()
						myproj4 = mysrs.proj4String()
						mywkt4326 = 'GEOGCS["WGS 84",DATUM["WGS_1984",SPHEROID["WGS 84",6378137,298.257223563,AUTHORITY["EPSG","7030"]],AUTHORITY["EPSG","6326"]],PRIMEM["Greenwich",0,AUTHORITY["EPSG","8901"]],UNIT["degree",0.01745329251994328,AUTHORITY["EPSG","9122"]],AUTHORITY["EPSG","4326"]]'
						#QMessageBox.information(self.iface.getMainWindow(),"Information",str(myproj4)   )

						#output kml file path for each layer
						mydestpath = os.path.abspath(os.path.join(str(mydir),str(myname)+".kml"))

						
						if (myproj4 != '' ): #process only vector layers with a well known srs
						#use ogr2ogr to create kml file
							if (str(myprovidername)=="postgres"): #postgresql
									mypglayerinfo = mysource.split('table')[0]
									myogr2ogr = 'ogr2ogr -a_srs EPSG:'+str(myepsg)+' -t_srs '+str(mywkt4326)+' -overwrite -f "KML" "'+str(mydestpath)+'" PG:"' + str(mypglayerinfo) + '" layer ' + str(myname)
									res=os.popen(myogr2ogr).readlines()

							elif (str(myprovidername)=="ogr"): #ogr
									myogr2ogr = 'ogr2ogr -a_srs "' + str(myproj4) + '" -t_srs '+str(mywkt4326)+' -overwrite -f "KML" "' + str(mydestpath) + '" "'+ str(mysource) +'" '
									res=os.popen(myogr2ogr).readlines()			

							else: #do nothing and warn the user
									QMessageBox.information(self.iface.getMainWindow(),"Information",str("Only postgres and ogr providers are supported") )
							
							QMessageBox.information(self.iface.getMainWindow(),"myogr2ogr",str(myogr2ogr) )
							#QMessageBox.information(self.iface.getMainWindow(),"res",str(res) )

						else:
							QMessageBox.information(self.iface.getMainWindow(),"Information","The layer " + str(myname) + " has an unknown spatial reference system." )
						
						#change progress bar value
						self.dlg.ui.progressBar.setValue(compteur)
				else:
				#warn the user
						QMessageBox.information(self.iface.getMainWindow(),"Information",str("Only vector layers are supported") )

			
			#Generates XHTML 1 / Javascript
			html = ['<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">\n']
			html.append('<html xmlns="http://www.w3.org/1999/xhtml">\n')
			html.append('<head>\n')
			html.append('<title>OGR2Layers</title>\n')
			
			# Save chosen Map Size
			mapSize = self.dlg.ui.mapSize.currentIndex()
			
			if (mapSize) == 0: #200x200
				html.append('<style>\n #map{width:400px;height:400px;}\n</style>\n')
			
			elif (mapSize) == 1: #400x400
				html.append('<style>\n #map{width:800px;height:600px;}\n</style>\n')
			
			elif (mapSize) == 2: #800x600
				html.append('<style>\n #map{width:100%;height:1024px;}\n</style>\n')
			
			
			#Call for OpenLayers 2.6 API on Metacarta servers
			html.append('<script src="http://www.openlayers.org/api/2.6/OpenLayers.js"></script>\n')
			html.append('<script type="text/javascript">\n')
			html.append('var map\n')
			html.append('function init(){\n')
			html.append("map = new OpenLayers.Map('map');\n")
			
			#Save chose Map Base Layer
			mapBaseLayer = self.dlg.ui.mapBaseLayer.currentIndex()
			
			if (mapBaseLayer) == 0: #OpenStreetMap WMS
				html.append('osm = new OpenLayers.Layer.WMS( "OpenStreetMap", ["http://t1.hypercube.telascience.org/tiles?", "http://t2.hypercube.telascience.org/tiles?", "http://t3.hypercube.telascience.org/tiles?", "http://t4.hypercube.telascience.org/tiles?"], {layers: "osm-4326", format: "image/png" } );\n')
				html.append('map.addLayer(osm);\n')
				
			elif (mapBaseLayer) == 1: #OpenLayers WMS
				html.append('olwms = new OpenLayers.Layer.WMS( "OpenLayers WMS", ["http://labs.metacarta.com/wms/vmap0"], {layers: "basic", format: "image/gif" } );\n')
				html.append('map.addLayer(olwms);\n')
				
			elif (mapBaseLayer) == 2: #OpenLayers WMS
				html.append('bmwms = new OpenLayers.Layer.WMS( "Demis WMS", ["http://www2.demis.nl/WMS/wms.asp?wms=WorldMap"], {layers: "Bathymetry,Countries,Topography,Hillshading,Builtup+areas,Coastlines,Waterbodies,Inundated,Rivers,Streams,Railroads,Highways,Roads,Trails,Borders,Cities,Settlements"} );\n')
				html.append('map.addLayer(bmwms);\n')
			
			for layer in layers:
				html.append('map.addLayer(new OpenLayers.Layer.GML("'+layer.name()+" "+'KML","'+layer.name()+'.kml"' +',{format: OpenLayers.Format.KML}));\n')
			
			#Layer Switcher active or not
			layerSwitcherActive = self.dlg.ui.layerSwitcherActive.currentIndex()
			
			if (layerSwitcherActive) == 0:
				html.append('var ls= new OpenLayers.Control.LayerSwitcher(); \n map.addControl(ls); \n ls.maximizeControl(); \n ')
				
			elif (layerSwitcherActive) == 1:
				html.append('\n')
				
			#User defined Bounds
			xmin = self.dlg.ui.lineEdit.text()
			xmax = self.dlg.ui.lineEdit_2.text()
			ymin = self.dlg.ui.lineEdit_3.text()
			ymax = self.dlg.ui.lineEdit_4.text()
			
			html.append('map.zoomToExtent(new OpenLayers.Bounds(' +str(xmin)+','+str(xmax)+','+str(ymin)+','+str(ymax)+'))\n};\n')
			html.append('</script>\n</head>\n')
			html.append('<body onload="init()">\n')
			
			#Generate H1 Map Title
			mapTitle = self.dlg.ui.mapTitle.text()
			html.append('<h1>'+ mapTitle +'</h1>\n')
			html.append('<div id="map"></div>\n')
			html.append('</body>\n')
			html.append('</html>\n')
			htmlfilename = str(mydir)+'/index.html'
			file = open(htmlfilename, "w")
			file.writelines(html)
			file.close()
			
			self.dlg.close()
			QMessageBox.information(self.iface.getMainWindow(),"Information",str("The OpenLayers Map has been created !") )
			
		
		else:
			QMessageBox.information(self.iface.getMainWindow(),"Warning", "Some errors occured before processing. Please check and resolve the following errors:\n" + str([str(x)+"\n" for x in control]) )
			self.dlg.close()
			self.run()

		
		
		
		
		
		
	def renderTest(self, painter):
		# QGIS Painter use for drawing to map canvas
		print "OGR2Layers: renderTest called!"
	
	